
package xelfi.browser.document;

import xelfi.browser.view.ObjectView;
import xelfi.browser.util.HMember;
import xelfi.browser.util.Sort;
import xelfi.editor.*;

import java.util.Vector;
import java.awt.Frame;
import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

public abstract class BrObject extends BrComponent implements HMember
{
    private XFile       xFile;
    public String       flags       = "";
    public Modifiers    modifiers;
    public BrPackage    Package;
    private MyPosition  positionBegin;
    private MyPosition  positionEnd;
    private MyPosition  positionBodyEnd;
    protected MyPosition  commentPositionB;
    protected MyPosition  commentPositionE;

    private int         projekt;    // patri k projektu,je MAIN???
    private int         methodNum   = 0;
    private int         variableNum = 0;
    private Vector      member      = new Vector(30,20);
    private boolean     opened      = false;
    public boolean      isLoaded    = false;

    BrObject(BrPackage pkg,DataInputStream is) throws IOException
    {
        positionBegin    = MyPosition.read(is);
        positionEnd      = MyPosition.read(is);
        commentPositionB = MyPosition.read(is);
        commentPositionE = MyPosition.read(is);
        Package          = pkg;
        name             = is.readUTF();
        XFile xf         = pkg.getXFile();
        xFile            = new XFile(xf.getPathStr(),xf.getPackage(),
                              is.readUTF());
        modifiers        = new Modifiers(is.readChar());
    }
    public BrObject(
                MyPosition pPositionB,
                MyPosition pPositionE,
                MyPosition pCommentPositionB,
                MyPosition pCommentPositionE,
                BrPackage pPackage,
                String pName,
                XFile pXFile,

                Modifiers modif)
    {
        positionBegin   = pPositionB;
        positionEnd     = pPositionE;
        commentPositionB = pCommentPositionB;
        commentPositionE = pCommentPositionE;
        Package         = pPackage;
        name            = pName;
        xFile           = pXFile;

        modifiers       = modif;
        return;
    }
    void save(DataOutputStream os) throws EndOfTextException,
        IOException
    {
        save1(os);
        int i,k = member.size();
        BrMember m;
        for(i=0;i<k;i++)
        {
            m = (BrMember)member.elementAt(i);
            if( m instanceof BrMethod )
            {
                os.writeByte('m');
                m.save(os);
            }else
            {
                os.writeByte('v');
                m.save(os);
            }
        }
        return;
    }
    void save1(DataOutputStream os) throws EndOfTextException,
        IOException
    {
        MyPosition.write(positionBegin,os);
        MyPosition.write(positionEnd,os);
        MyPosition.write(commentPositionB,os);
        MyPosition.write(commentPositionE,os);
        os.writeUTF(name);
        os.writeUTF(getXFile().getName());
        os.writeChar(modifiers.getInt());
    }
    // metody od BrComponent
    /**
     * This method return name with modifiers and other
     * parameters.
     * @param bs Actual settings.
     * @param depth If this component listbox is hierarched
     * this parameter stores information about components depth
     * in this tree.
     * @return Full name
     */
    public String getFullName(BrSettings bs,int depth)
    {
        StringBuffer s = new StringBuffer();
        if( bs.ObjShowModif )
        {
            s.append(flags);
            //s.append(" ");
        }
        if( depth > 0 )
        {
            int i,k = (depth - 1) * 2;
            for(i=0;i<k;i++)
                s.append(" ");
        }
        s.append(name);
        return new String(s);
    }
    public String getText()
    {
        return "";
    }

    // metody od HMember
    public String getParent()
    {
        return "xxx";
    }

    // ostatni
    public XFile getXFile()
    {
        return xFile;
    }
    public String getNameWithPackage()
    {
        return Package.name + "." + name;
    }
    public boolean addComponent(BrComponent c)
    {
        member.addElement(c);
        if( c instanceof BrMethod )
            methodNum++;
        else
        if( c instanceof BrVariable )
            variableNum++;
        else
            System.out.println("BrInterface.addComponent snazim se pridat cosi divneho!!! :" + c);
        return true;
    }
    public void removeComponent(BrComponent c)
    {
        if( member.removeElement(c) )
        {
            if( c instanceof BrMethod )
                methodNum--;
            else
            if( c instanceof BrVariable )
                variableNum--;
            else
                System.out.println("BrInterface.addComponent snazim " +
                    "se pridat cosi divneho!!! :" + c);
        }
        return;
    }
    public void addComponentBehind(BrComponent c,BrComponent cc)
    {
        int i = member.indexOf(cc);

        member.insertElementAt(c,i+1);

        if( c instanceof BrMethod )
            methodNum++;
        else
        if( c instanceof BrVariable )
            variableNum++;
        else
            System.out.println("BrInterface.addComponent snazim " +
                "se pridat cosi divneho!!! :" + c);
        return;
    }
    /**
     * This method returns a number of actually visible
     * subcomponents.
     * @param bs Actuall settings
     * @return Number of visible subcomponents
     */
    public int getComponentsNumber(BrSettings bs)
    {
        int i = 0;
        if( bs.MultipleSelectioned && (bs.MemSort != 2) )
            i = 1;
        return (bs.MemMethod?methodNum:0) +
               (bs.MemVariable?variableNum:0) + i;
    }
    /**
     * This method returns actually visible subcomponents (methods
     * and variables) of this component. This components are stored
     * to the given array form @see from index.
     * @param bs Actual settings.
     * @param c The array of the components.
     * @param from First non occupied position.
     * @return The index of the first non occupied position
     * past adding components to the array
     */
    public int getComponents(BrSettings bs,BrComponent[] c,int from)
    {
        if( bs.MultipleSelectioned && (bs.MemSort != 2) )
            c[from++] = new BrComponent(name);

        BrComponent p;
        int i,od = from,k = member.size();
        for(i=0;i<k;i++)
        {
            p = (BrComponent)member.elementAt(i);
            if( bs.MemMethod && (p instanceof BrMethod) )
                c[from++] = p;
            else
            if( bs.MemVariable && (p instanceof BrVariable) )
                c[from++] = p;
        }
        if( bs.MemSort == 1 )
            c = (BrComponent[])Sort.sort(c,od,from);
        return from;
    }
    void fileIsOpened(xelfi.editor.Document d)
        throws EndOfTextException
    {
        //System.out.println("fileIsOpened ");

        if( isLoaded )
            return;
        positionBegin.open(d);
        positionEnd.open(d);
        if( positionBodyEnd != null )
            positionBodyEnd.open(d);
        if( commentPositionB != null )
        {
            commentPositionB.open(d);
            commentPositionE.open(d);
        }
        int i,k = member.size();
        for(i=0;i<k;i++)
            ((BrMember)member.elementAt(i)).open(d);
        isLoaded = true;
        //System.out.println("fileIsOpened end");

        return;
    }
    void fileIsClosed()
    {
        if( !isLoaded )
            return;
        isLoaded = false;
        Document d = positionBegin.getPosition().getDocument();
        positionBegin.close();
        positionEnd.close();
        if( positionBodyEnd != null )
            positionBodyEnd.close();
        if( commentPositionB != null )
        {
            commentPositionB.close();
            commentPositionE.close();
        }
        int i,k = member.size();
        for(i=0;i<k;i++)
            ((BrMember)member.elementAt(i)).close();
        return;
    }
    public void setBodyEnd(MyPosition position)
    {
        positionBodyEnd = position;
    }
    public void setEnd(MyPosition position)
    {
        positionEnd = position;
    }
    public void setBegin(MyPosition position)
    {
        positionBegin = position;
    }
    public Position getBegin()
    {
        return positionBegin.getPosition();
    }
    public Position getEnd()
    {
        return positionEnd.getPosition();
    }
    public Position getBodyEnd()
    {
        return positionBodyEnd.getPosition();
    }
    /**
     * This method opens view for this object.
     * @param f Parent frame
     */
    public void openView(Frame f)
    {
        ObjectView v = new ObjectView(f,this);
    }
    /**
     * This method returns true, if selecting of this component
     * is allowed.
     * @return true
     */
    public boolean selectable()
    {
        return true;
    }
    public String getHint()
        throws IOException
    {
        if( commentPositionB == null )
        {
            return "This class \n\n@see \n@version 1.0\n" +
                "@author \n";
        }else
        {
            Source src = new Source(getXFile());
            InputStream is = (new Block(commentPositionB.getPosition(),
                commentPositionE.getPosition())).
                getInputStream();
            StringBuffer b = new StringBuffer();
            int c;
            while( (c = is.read()) >= 0 ) b.append((char)c);
            String s = new String(b);

            return hintToString(s);
        }
    }
    public void setHint(String hint)
        throws IOException,NotAllowedException
    {
        if( commentPositionB == null )
        {
            int i,indent;
            hint = strToHint(hint,indent = positionBegin.getPosition().
                getColumn());
            Position p,pom;
            Source s = new Source(getXFile());
            pom = getBegin();
            p = pom.insert("/**\n" + hint);
            StringBuffer b = new StringBuffer();
            b.append('\n');
            for(i=0;i<indent;i++) b.append(' ');
            commentPositionE = new MyPosition(
                pom.insert(new String(b)));
            commentPositionB = new MyPosition(p.next(3));
        }else
        {
            hint = strToHint(hint,commentPositionB.getPosition().
                getColumn()-2);
            Position p,pom;
            Source s = new Source(getXFile());
            (new Block(pom = commentPositionB.getPosition(),
                commentPositionE.getPosition())).delete();
            p = pom.insert("\n" + hint);
            commentPositionB = new MyPosition(p);
            commentPositionE = new MyPosition(pom);
        }
        return;
    }
}





















