package xelfi.browser.document;
import java.io.InputStream;
import java.io.IOException;
import xelfi.browser.util.PositionedInputStream;

import xelfi.editor.EndOfTextException;

public class XStreamTokenizer
{
    boolean commentR = true;
    public void commentReport(boolean b)
    {
        commentR = b;
    }
    private InputStream is;
    private char buf[];
    private int peekc = ' ';
    private boolean pushedBack;
    private boolean forceLower;
    /** The line number of the last token read */
    private int LINENO = 0;
    private int col = 0;
    //private int oldCol = 0;

    private boolean eolIsSignificantP = false;
    private boolean slashSlashCommentsP = false;
    private boolean slashStarCommentsP = false;


    private byte ctype[] = new byte[256];
    private static final byte CT_WHITESPACE = 1;
    private static final byte CT_DIGIT = 2;
    private static final byte CT_ALPHA = 4;
    private static final byte CT_QUOTE = 8;
    private static final byte CT_COMMENT = 16;

//    private boolean editor;

    public int ttype;

    /**
     * The End-of-file token.
     */
    public static final int TT_EOF = -1;

    /**
     * The End-of-line token.
     */
    public static final int TT_EOL = '\n';

    /**
     * The number token.  This value is in nval.
     */
    public static final int TT_NUMBER = -2;

    /**
     * The word token.  This value is in sval.
     */
    public static final int TT_WORD = -3;
    public static final int TT_SLASHSTAR = -4;
    public static final int TT_SLASHSLASH = -5;
    public static final int TT_TEXT = -6;


    /**
     * The Stream value.
     */
    public String sval;

    /**
     * The number value.
     */
    public double nval;

    //private PositionedInputStream               Qinput;
    private xelfi.editor.DocumentInputStream    Jinput;
    private boolean                             fromFile;
//    private MyPosition     positionBegin    = new MyPosition(0,0);
//    private MyPosition     positionEnd      = new MyPosition(0,0);
    private MyPosition     begin            = new MyPosition(0,0);
    private MyPosition     end              = new MyPosition(0,0);
    private MyPosition     oldBegin         = new MyPosition(0,0);
    private MyPosition     oldEnd           = new MyPosition(0,0);
    private MyPosition     middle           = new MyPosition(0,0);
    private MyPosition     oldMiddle        = new MyPosition(0,0);
    private MyPosition     commentPosB      = new MyPosition(0,0);
    private MyPosition     commentPosE      = new MyPosition(0,0);

    private boolean        comment          = false;

    public XStreamTokenizer (InputStream I)
    {
        is = I;
        if( I instanceof xelfi.editor.DocumentInputStream )
        {
            Jinput = (xelfi.editor.DocumentInputStream)I;
            fromFile = false;
        }else
            fromFile = true;
        Init();
    }
    private void Init()
    {
        buf = new char[20];
        byte ct[] = ctype;
        int i;
        wordChars('a', 'z');
        wordChars('A', 'Z');
        wordChars('_','_');
        //wordChars(128 + 32, 255);
        whitespaceChars(0, ' ');
        commentChar('/');
        quoteChar('"');
        quoteChar('\'');
        parseNumbers();
    }
    private void wordChars(int low, int hi)
    {
        if( low < 0 )
            low = 0;
        if( hi > ctype.length )
            hi = ctype.length;
        while( low <= hi )
            ctype[low++] |= CT_ALPHA;
    }

    private void whitespaceChars(int low, int hi)
    {
        if (low < 0)
            low = 0;
        if (hi > ctype.length)
            hi = ctype.length;
        while (low <= hi)
            ctype[low++] = CT_WHITESPACE;
    }

    private void ordinaryChars(int low, int hi)
    {
        if (low < 0)
            low = 0;
        if (hi > ctype.length)
            hi = ctype.length;
        while (low <= hi)
            ctype[low++] = 0;
    }

    private void ordinaryChar(int ch)
    {
        ctype[ch] = 0;
    }

    private void commentChar(int ch)
    {
        ctype[ch] = CT_COMMENT;
    }

    private void quoteChar(int ch)
    {
        ctype[ch] = CT_QUOTE;
    }
    private void parseNumbers()
    {
        for (int i = '0'; i <= '9'; i++)
            ctype[i] |= CT_DIGIT;
        ctype['.'] |= CT_DIGIT;
        ctype['-'] |= CT_DIGIT;
    }
    public void eolIsSignificant(boolean flag)
    {
        eolIsSignificantP = flag;
    }
    public MyPosition getPosition()
    {
        try{
            if( pushedBack )
                return oldEnd;
                //return positionEnd;
            else
            {
                if( fromFile )
                    return new MyPosition(LINENO,col);
                else
                    return new MyPosition(Jinput.getPosition().
                           previousChar());
                    /*return new MyPosition((peekc == '\n' ||
                        peekc == '\r') ? Jinput.getPosition() :
                        Jinput.getPosition().previousChar());*/
            }
        }catch(EndOfTextException e)
        {
            return new MyPosition(Jinput.getPosition());
        }
    }
    public MyPosition getCommentPositionBegin()
    {
        return commentPosB;
    }
    public MyPosition getCommentPositionEnd()
    {
        return commentPosE;
    }
    public MyPosition getPreviousEnd()
    {
        if( pushedBack )
            return oldBegin;
        else
            return begin;
        //return positionBegin;
    }
    public MyPosition getMiddle()
    {
        if( pushedBack )
            return oldMiddle;
        else
            return middle;
        /*if( !pushedBack )
            return middle;
        else
            return null;*/
    }
    public MyPosition getNextBegin()
        throws IOException
    {
        nextToken();
        MyPosition p = getMiddle();
        pushBack();
        return p;
/*        if( pushedBack )
            return middle;
        else
        {
            positionEnd = getPosition();
            readWhitespace();
            return getPosition();
        }*/
    }

    private void readWhitespace()
        throws IOException
    {
        int c = peekc;// last read

        if( c < 0 )
        {
            peekc = c;
            return;
        }

        int ct = c < 256 ? ctype[c] : CT_ALPHA;

        while( (ct & CT_WHITESPACE) != 0 )
        {//                                 WHITESPACE!!!!!!!
            if( c == '\r' )
            {
               LINENO++;
               col = 0;
               c = is.read();
               if( c == '\n' )
                   c = is.read();
            }else
            if( c == '\n' )
            {
                LINENO++;
                c = is.read();
                col = 0;
            }else
            if( c == '\t' )
            {
                col += ((c = (3 - (col % 4))) == 0) ? 4 : c;///!!!!!!!!!!!
                c = is.read();
            }else
            {
                c = is.read();
                col++;
            }
            if( c < 0 )
            {
                peekc = c;
                return;
            }
            ct = c < 256 ? ctype[c] : CT_ALPHA;
        }//WHILE Whitespace
        peekc = c;
        return;
    }
    /*private int readEol(int c) throws IOException
    {
        if( c == '\r' )
        {
            LINENO++;
            col = 0;
            c = is.read();
            if( c == '\n' )
                c = is.read();
        }else
        if( c == '\n' )
        {
            LINENO++;
            c = is.read();
            col = 0;
        }
        return c;
    }*/
    public int readTo(int ch1,int ch2) throws IOException
    {
        StringBuffer s = new StringBuffer();
        int i = 0 , c, depth = 0;
        boolean lastWord = false;
        c = nextToken();
        while( ((c != ch1) && (c != ch2) || (depth > 0)) &&
            (c != TT_EOF) )
        {
           /*if( i >= buf.length )
           {
               char nb[] = new char[buf.length * 2];
               System.arraycopy(buf, 0, nb, 0, buf.length);
               buf = nb;
           }
           if( c >= 0 ) buf[i++] = (char) c;
           c = is.read();
           if( c == TT_EOF )
               return ttype = TT_EOF;*/
           switch( c )
           {
           case TT_WORD:
               if( lastWord )  s.append(' ').append(sval);
               else s.append(sval);
               lastWord = true;
               break;
           case TT_NUMBER:
               if( lastWord )  s.append(' ').append(nval);
               else s.append(nval);
               lastWord = true;
               break;
           case TT_TEXT:
               if( lastWord )  s.append(' ').append((char)ttype).
                   append(sval).append((char)ttype);
               else s.append((char)ttype).append(sval).
                   append((char)ttype);
               lastWord = true;
               break;
           case '(':
               depth++;
               s.append((char)c);
               lastWord = false;
               break;
           case ')':
               depth--;
               s.append((char)c).append(' ');
               lastWord = false;
               break;

           default:
               s.append((char)c);
               lastWord = false;
           }
           c = nextToken();
        }
        sval = new String(s);
        /*if( forceLower )
           sval = sval.toLowerCase();*/
        return ttype = c;
    }
    public int nextToken() throws IOException
    {
        if( pushedBack )
        {
            pushedBack = false;
            return ttype;
        }
        sval = null;

        if( !comment )
        {
            oldEnd = getPosition();
            begin = oldEnd;
            oldBegin = begin;
            oldMiddle = middle;
            commentPosB = null;
        }

        readWhitespace();
        int c = peekc;

        if( c < 0 )
            return ttype = TT_EOF;//        EOF!!!!!!!!!!!!

        //if( !comment )
        middle = getPosition();

        comment = false;

        int ct = c < 256 ? ctype[c] : CT_ALPHA;

        if( (ct & CT_DIGIT) != 0 )
        {//                                 CISLA!!!!!!!!!!!
            boolean neg = false;
            if( c == '-' )
            {
               c = is.read();
               col++;
               if( c != '.' && (c < '0' || c > '9') )
               {
                   peekc = c;
                   return ttype = '-';
               }
               neg = true;
            }
            double v = 0;
            int decexp = 0;
            boolean digit = false;
            int seendot = 0;//pocet cislic za .
            while( true )// 12.332
            {
               if( c == '.' && seendot == 0 )
                   seendot = 1;
               else
               if( '0' <= c && c <= '9' )
               {
                   v = v * 10 + (c - '0');
                   decexp += seendot;
                   digit = true;
               }else
                   break;
               c = is.read();
               col++;
            }// v = 12332  decexp = 3
            peekc = c;
            if( !digit && (seendot == 1) )
            {
                return ttype = '.';
            }
            if( decexp != 0 )
            {
               double denom = 10;
               decexp--;
               while( decexp > 0 )
               {
                   denom *= 10;
                   decexp--;
               }
               /* do one division of a likely-to-be-more-accurate number */
               v = v / denom;
            }
            nval = neg ? -v : v;
            return ttype = TT_NUMBER;
        }
        if( (ct & CT_ALPHA) != 0 )
        {//                                 WORD!!!!!!!!!!
            int i = 0;
            do{
               if( i >= buf.length )
               {
                   char nb[] = new char[buf.length * 2];
                   System.arraycopy(buf, 0, nb, 0, buf.length);
                   buf = nb;
               }
               buf[i++] = (char) c;
               c = is.read();
               col++;
               ct =  c == '.' ? 0 : c < 0 ? CT_WHITESPACE : c < 256 ?
                   ctype[c] : CT_ALPHA;

            }while( ((ct & (CT_ALPHA | CT_DIGIT)) != 0) );
            peekc = c;
            sval = String.copyValueOf(buf, 0, i);
            /*if( forceLower )
               sval = sval.toLowerCase();*/
            return ttype = TT_WORD;
        }
        if ((ct & CT_QUOTE) != 0)
        {//                     NACITA  ""  NEBO ''!!!!!!!!!!!!!!!!
            ttype = c;
            int i = 0;
            while( (c = is.read()) >= 0 && c != ttype && c != '\n' &&
                c != '\r' )
            {
                col++;
                if( c == '\\' )
                {
                    col++;
                    switch( c = is.read() )
                    {
                    case '\\':
                        c = '\\';
                        break;
                    case '\'':
                        c = '\'';
                        break;
                    case '\"':
                        c = '\"';
                        break;
                    case 'a':
                        c = 0x7;
                        break;
                    case 'b':
                        c = '\b';
                        break;
                    case 'f':
                        c = 0xC;
                        break;
                    case 'n':
                        c = '\n';
                        break;
                    case 'r':
                        c = '\r';
                        break;
                    case 't':
                        c = '\t';
                        break;
                    case 'v':
                        c = 0xB;
                        break;
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                        c = c - '0';
                        int c2 = is.read();
                        col++;
                        if( '0' <= c2 && c2 <= '7' )
                        {
                            c = (c << 3) + (c2 - '0');
                            c2 = is.read();
                            col++;
                            if( '0' <= c2 && c2 <= '7' )
                                 c = (c << 3) + (c2 - '0');
                            else
                                 peekc = c;
                        }else
                            peekc = c;
                        break;
                    }//SWITCH
                }//if
                if( i >= buf.length )
                {
                    char nb[] = new char[buf.length * 2];
                    System.arraycopy(buf, 0, nb, 0, buf.length);
                    buf = nb;
                }
                buf[i++] = (char) c;
            }//WHILE
            col++;
            peekc = is.read();
            sval = String.copyValueOf(buf, 0, i);
            return TT_TEXT;
        }
        if( c == '/' )
        {//                 KOMENTAR!!!!!!!!!!!!!!!!!!!!!!!!
            c = is.read();
            col++;
            if( c == '*' )
            {//  /*
                int prevc = is.read();
                boolean superComment = false;
                col++;
                if( prevc == '*' )
                {//  /**
                    prevc = is.read();
                    col++;
                    if( prevc == '/' )
                    {//  /**/
                        peekc = ' ';
                        if( commentR )
                        {
                            return TT_SLASHSTAR;
                        }else
                        {
                            comment = true;
                            return nextToken();
                        }
                    }
                    superComment = true;
                    peekc = prevc;
                    commentPosB = getPosition();
                }
               //int prevc = 0;
               if( prevc == '\n' )
               {
                   LINENO++;
                   col = 0;
               }
               while( (c = is.read()) != '/' || prevc != '*' )
               {
                   col++;
                   if (c == '\n')
                   {
                      LINENO++;
                      col = 0;
                   }
                   if (c < 0)
                      return ttype = TT_EOF;
                   if( eolIsSignificantP )
                   {
                      return ttype = TT_EOL;
                   }
                   prevc = c;
               }
               col++;
               peekc = is.read();
               if( superComment )
                   commentPosE = getPosition();
               if( commentR ) return TT_SLASHSTAR;
               else
               {
                   comment = true;
                   return nextToken();
               }
            }else
            if( c == '/' )
            {

               while( (c = is.read()) != '\n' && c != '\r' && c >= 0 )
                   col++;
               peekc = c;
               if( commentR ) return TT_SLASHSLASH;
               else
               {
                   comment = true;
                   return nextToken();
               }
            }else
            {
               peekc = c;
               return ttype = '/';
            }
        }
        peekc = is.read();
        col++;
        return ttype = c;
    }
    public void pushBack()
    {
        if( pushedBack )
            System.out.println("XStreamTokenizer error: "
                + "pushBack twice !!!");
        pushedBack = true;
    }
    public int lineno()
    {
        return LINENO;
    }
    public String toString()
    {
        String ret;
        switch (ttype)
        {
          case TT_EOF:
            ret = "EOF";
            break;
          case TT_EOL:
            ret = "EOL";
            break;
          case TT_WORD:
            ret = sval;
            break;
          case TT_NUMBER:
            ret = "n=" + nval;
            break;
          default:
            {
           char s[] = new char[3];
           s[0] = s[2] = '\'';
           s[1] = (char) ttype;
           ret = new String(s);
           break;
            }
        }
        return "Token[" + ret + "], line " + LINENO;
    }
}


















