/**
 * LocalsView
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

import sun.tools.debug.*; // needs TCP/IP
//import symantec.tools.debug.*; // doesn't need TCP/IP

/**
 * This is a view on the data given by the Debugger document.
 * It is a view on local variables of the currently executed method in the currently
 * debugged thread.
 */

public class LocalsView extends TopDialog
{
	private Debugger document;
	private List list;

	static final int KEY_ENTER = 10;

	static final int HINT_COMPLETE_UPDATE = 0;

	/**
	 * LocalsView constructor.
	 *
	 * debugger - the document for this view
	 */
	LocalsView(Debugger debugger)
	{
		super(TopLevel.getMenu(), false);
		setTitle("Locals");
		//super("Locals");
		setBackground(Color.lightGray);

		document = debugger;
		list = new List();
		add("Center", list);
	}

	/**
	 * This method is called when the content of the view must be
	 * updated.
	 */
	void update(int hint, int index)
	{
		if (!isVisible())
			return;

		if (hint != HINT_COMPLETE_UPDATE)
		{
			System.out.println("unsupported hint");
			return;
		}

		RemoteStackVariable rsv[] = document.getStackVariables();

		if (rsv == null)
		{
			list.clear();
			return;
		}

		int indexOfSelected = list.getSelectedIndex();
		if (indexOfSelected != -1)
			list.deselect(indexOfSelected);

		for(int i = 0; i < rsv.length; i++)
		{
			String line = rsv[i].getName();

			line += ": "+document.getCurrentValue(rsv[i].getName());

			if (i > list.countItems()-1)
				list.addItem(line);
			else
				if (!list.getItem(i).equals(line))
					list.replaceItem(line, i);
		}

		if (rsv.length < list.countItems())
			list.delItems(rsv.length, list.countItems()-1);
	}

	/**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		list.requestFocus();
		document.localsViewShown();
	}
}
