/**
 * WatchDialog
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

/**
 * This dialog is used to enter the watch name.
 * Result from the dialog is send via parent.action() to the dialog's
 * parent.
 */

class WatchDialog extends TopDialog
{
	static final int ID_OK = 1;
	static final int ID_CANCEL = 0;

	private WatchesView parent;

	private Label label;
	private TextField edit;
	private Button okButton;
	private Button cancelButton;

	/**
	 * WatchDialog constructor.
	 */
	public WatchDialog(WatchesView parent, String title, String watchName)
	{
		super(TopLevel.getMenu(), false);
		setTitle(title);
		//super(title);

		this.parent = parent;

		setBackground(Color.lightGray);

		resize(insets().left + insets().right + 355, insets().top + insets().bottom + 171);
		setResizable(false);

		label=new Label("Name");
		add(label);
		label.reshape(insets().left + 18,insets().top + 36,90,16);

		edit=new TextField(watchName, 31);
		edit.setBackground(Color.lightGray);

		add(edit);
		edit.reshape(insets().left + 18,insets().top + 60,315,32);

		okButton=new Button("OK");
		add(okButton);
		okButton.reshape(insets().left + 89,insets().top + 112,70,28);

		cancelButton=new Button("Cancel");
		add(cancelButton);
		cancelButton.reshape(insets().left + 191,insets().top + 112,69,28);

		centerWindow();
	}

	/**
	 * Tahle metoda by mela byt implementovana jiz v TopFrame.
	 * FILIP
	 */
	void centerWindow()
	{
	}

	/**
	 * My action() implementation.
	 */
	public boolean action(Event event, Object what)
	{
		if ((event.target == okButton)
			|| (event.target == edit))
		{
			WatchDialogResult result;

			if (edit.getText().compareTo("") == 0)
				return true;

			result = new WatchDialogResult();
			result.name = edit.getText();
			result.returnCode = ID_OK;
			hide();
			parent.action(null, result);
			return true;
		}
		if (event.target == cancelButton)
		{
			WatchDialogResult result = new WatchDialogResult();

			result.returnCode = ID_CANCEL;
			hide();
			parent.action(null, result);
			return true;
		}
		return super.action(event, what);
	}

	/**
	 * My handleEvent() implementation.
	 */
	public boolean handleEvent(Event event)
	{
		if (event.id == Event.WINDOW_DESTROY)
		{
			WatchDialogResult result = new WatchDialogResult();

			hide();
			result.returnCode = ID_CANCEL;
			parent.action(null, result);
			return true;
		}
		return super.handleEvent(event);
	}

        /**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		edit.requestFocus();
	}
}
