/**
* Xelfi
*/

package xelfi.design.components.xawt;

import java.awt.*;
import java.io.*;
import xelfi.design.components.*;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.design.components.property.*;
import xelfi.design.util.*;
import xelfi.util.XelfiPath;

/**
* This is the first non-visual component (except menu).
* It is a design class for class Timer, which is class that once or periodically sends 
* message to its owner with given period time.
* It has 3 properties:
*	DelayTime	...	time in ms for the timer
*	Cycle		...	if false then the timer ticks oly once/ if true then it works periodically
*	AutoStart	...	if true then the timer is strted right after it is created
*	(or better the code for starting it is generated right after the creation code)
* @version 	2.13, 15 Mar 1997
* @author 	Ian Formanek, Petr Hamernik
*/

public class XTimer implements XelfiDesignComponent {
	private static int propertyCount = 3;
	private static int eventCount = 1;
	private String events[] = new String[eventCount];
	private String name = "";
	private Image icon = null;
	private XelfiDesignForm parentForm = null;

	private int delayTime = 1000;
	private boolean cycle = true;
	private boolean autoStart = true;
	
//////////////////
// constructors //
//////////////////

	public XTimer() { 
		loadIcon();
	}

	public void setParentForm(XelfiDesignForm form) {
		parentForm = form;
	}

	public XelfiDesignForm getParentForm() {
		return parentForm;
	}

	public String getName() { return name; }
	
	public int setDesignName(String aName) { name = aName; return XFDesignConstants.RSP_OK; }
	
	public int getPropertyCount() { return propertyCount;	}
	
	public String[] getPropertyNames() {
		String list[] = new String[propertyCount];
		list[0] = "Delay Time";
		list[1] = "Cycle";
		list[2] = "Auto Start";
		return list;
	}	

	public Object getProperty(int index) throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: return new Integer(delayTime);
			case 1: return new Boolean(cycle);
			case 2: return new Boolean(autoStart);
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String getPropertyString(int index) throws 
		XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: return XIntegerPropertyType.viewString(getProperty(index)); 
			case 1:
			case 2: return XBooleanPropertyType.viewString(getProperty(index));
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: return XFDesignConstants.PVT_INPUT_INT; 
			case 1: 
			case 2: return XFDesignConstants.PVT_CHECK;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}


	public int setProperty(int index, Object value) throws 	
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException 
	{ 
		int ret = XFDesignConstants.RSP_OK;
		int regen = XFDesignConstants.GEN_INIT;
		
		switch (index) { 
			case 0: if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
					delayTime = ((Integer)value).intValue();
					if (delayTime <=0) {
						delayTime = 1000;
						ret = XFDesignConstants.RSP_DIFF_VALUE;
					}
					break;
			case 1: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					cycle = ((Boolean)value).booleanValue();
					break;
			case 2: if (!(value instanceof Boolean)) throw new XFPropertyBadValueTypeException();
					autoStart = ((Boolean)value).booleanValue();
					break;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
		if (parentForm != null)
			parentForm.regenerateCode(this, regen);
		
		return ret;
	}
			
	public void specialPropertyInput(Frame parent, int index) throws 	
		XFPropertyIndexOutOfBoundsException, 
		XFPropertyNoSpecialInputException { 
		if ((index < 0) || (index >= propertyCount)) throw new XFPropertyIndexOutOfBoundsException();
		switch (index) { 
			default: throw new XFPropertyNoSpecialInputException();
		}
	}

	
	public boolean parentUpdate() { return false; }

	public int getEventCount() { return eventCount; }
	
	public String[] getEventNames() {
		String list[] = new String[eventCount];
		list[0] = new String("OnTime"); 
		return list;
	}

	public String getEvent(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount)) {
			if (events[index] == null)
				return null;
			else
				return new String(events[index]); 
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}		
		
	public void setEvent(int index, String value) throws 
		XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount)) {
			events[index] = value;
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}		

	public Image getIcon() {
		return icon; 
	}

	private void loadIcon() {
		String sep = new String(System.getProperty("file.separator"));
		StringBuffer s = new StringBuffer(XelfiPath.get() + sep + 
			"design" + sep + "components" + sep + "xawt" + sep + "timer.gif");
		icon = Toolkit.getDefaultToolkit().getImage(new String(s));
	}
	
	public String getClassName() { return "Timer"; }
	
	public String[] neededImports() {
		String list[] = new String[1];
		list[0] = "xelfi.awt.Timer";
		return list;
	}

	public String generateCode(int what) {
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "(this, " + delayTime + ", "+cycle+");\n";
				break;
			case XFDesignConstants.GEN_SETPROPS:
				if (autoStart) 
					gen = name + ".start();\n";
				break;			
		}
		return gen;
	}

	public String generateEventCondition(int index) throws 
	XFPropertyIndexOutOfBoundsException {
		if (index==0){
			return XFDesignOptions.eventVarName+".target == "+name;
// Note: if the component	had multiple events then the condition would look like:
//	      evt.target == name && "NameOfEvent".equalsIgnoreCase((String)evt.arg)
// where evt is (in this example only) result of XFDesignOptions.eventVarName
// and the NameOfEvent differentiates between different events
// This scheme must be, of course, supported by the component class (it must create the event the way,
// that into event.target it puts reference to itself (this) and into event.arg string name of the event)
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}
	
	public void setSelection(int aSelection) {}

	public int getSelection() { return 0; }

	public void loadFromStream(DataInput stream) throws	IOException {
		name = stream.readUTF();
		delayTime = XIntegerPropertyType.loadFromStream(stream).intValue();
		cycle = XBooleanPropertyType.loadBFromStream(stream);
		autoStart = XBooleanPropertyType.loadBFromStream(stream);
	}
	
	public void saveToStream(DataOutput stream) throws IOException {
		stream.writeUTF(name);
		XIntegerPropertyType.saveToStream(new Integer(delayTime), stream);
		XBooleanPropertyType.saveToStream(cycle, stream);
		XBooleanPropertyType.saveToStream(autoStart, stream);
	}
}	