/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.Enumeration;
import java.util.NoSuchElementException;
/**
* Enumeration class that takes enumeration of Documents and
* creates enumeration of Sources.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class SourceEnumeration extends Object implements Enumeration {
  /** enumeration of loaded documents */
  private Enumeration en;

  /** only modified */
  private boolean modified;

  /** next element */
  private LoadedDocument next = null;
  
  /** New enumeration of sources.
  * @param en enumeration of loaded documents
  * @param mod true if we should enumerate only modified documents
  */
  public SourceEnumeration (Enumeration en, boolean mod) {
    this.en = en;
    modified = mod;
    getNext ();
  };
  
  /** New enumeration of sources.
  * @param en enumeration of loaded documents
  */
  public SourceEnumeration (Enumeration en) {
    this (en, false);
  };

  /** Loads next element to the next field */
  void getNext () {
    try {
      do {
        next = (LoadedDocument)en.nextElement ();
        // if modified then wait for the modified one
      } while (modified && !next.isModified ());
    } catch (NoSuchElementException ex) {
      next = null;
    };
  };

  /** has next element?
  * @return true if there is next element in enumeration
  */
  public boolean hasMoreElements () {
    return next != null;
  };

  /** Next element.
  * @return next element in enumeration
  * @exception NoSuchElementException if there is no next element
  */
  public Object nextElement () {
    if (next == null) {
      throw new NoSuchElementException ();
    } else {
      try {
        // creates source for the next element
        return new Source (next);
      } finally {
        // finds new next element
        getNext ();
      }
    }
  }
}

