/**
* The xelfi Editor
*/

package xelfi.editor;

/**
* The stack with undo events.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class Undo extends Object implements ActionController {
  /** the maximum undo operations */
  private static final int MAX_UNDO = 512;
  
  /** the stack */
  private UndoEvent[] stack = new UndoEvent[MAX_UNDO];

  /** first */
  private int first = 0;

  /** last */
  private int last = 0;
  
  /** if true then this object preforms undo */
  private boolean in = false;

  /** Forbids construction outside this class */
  private Undo () {
  }

  /** Creates new undo and registers it in the document.
  * @param d the document
  */
  private Undo (Document d) {
    d.addActionController (this);
  }
  
  /** Constructs new undo object for specified document or finds the previously
  * created one.
  * @param d the document
  * @return the undo
  */
  public static Undo getUndo (Document d) {
/*    Event e = new Event (null, Document.GET_UNDO, null);
    // searching for the undo object for a document
    if (d.handleEvent (null, e) && e.target != null) {
      return (Undo)e.target;
    } else {
      // create new one
      Undo u = new Undo (d);
      return u;
      }
      */
    return null;
  }

  /** Undoes one action.
  * @return the position of undo
  * @exception NotAllowedException if it is not possible to do the undo
  */
  public synchronized Position undo () throws NotAllowedException {
    if (last == first) {
      throw new NotAllowedException ();
    } else {
      in = true;
      try {
        Position p;
        if (first-- == 0) first = MAX_UNDO - 1;
        if ((p = stack[first].undo ()) == null) {
          // if it is not posible to undo the action then
          // empty the stack
          last = first = 0;
          throw new NotAllowedException ();
        } else {
          return p;
        }
      } finally {
        in = false;
      }
    }
  }

  /** Adds new undo event.
  * @param e the event
  */
  private synchronized void addEvent (UndoEvent e) {
    stack[first] = e;
    if (++first == MAX_UNDO) first = 0;
    if (first == last) {
      if (++last == MAX_UNDO) last = 0;
    }
  }

  /** This method is called before every insert to any position
  * For multiple inserts (of arrays or Strings) is called only
  * once at begining.
  *
  *
  * @param p the position method insert will insert to
  * @param info optional info sent to insert or null if nothing has been sent
  * @exception NotAllowedException if this insert is not allowed
  */
  public void allowInsert (Position p, Object info) throws NotAllowedException {
  }
  
  /** This method is called before every delete of single character at any position.
  *
  * @param p the position method delete will delete at
  * @param info optional info sent to delete
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowDelete (Position p, Object info) throws NotAllowedException {
  }

  /** This method is called before every delete of a block (more characters).
  * @param from the first position to delete
  * @param to the last (excluded) position
  * @param info the info
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowMultiDelete (Position from, Position to, Object info)
  throws NotAllowedException {
  }
  
  /** Notifies that an insert has been done at position p.
  * @param n the begin of inserted text
  * @param p the end of inserted text
  */
  public void notifyInsert (Position n, Position p) {
    if (!in) {
      addEvent (new UndoInsert (n, p));
    }
  }
  
  /** Notifies a delete
  * @param p the position where a delete occures
  * @param s the string with characters that has been deleted
  */
  public void notifyDelete (Position p, String s) {
    if (!in) {
      addEvent (new UndoDelete (p, s));
    }
  }
  
  /** Handle event allows sending some predefined messages to position
  * and by throw the document to ActionController
  * @param p the position a message is sent to
  * @param e the event (the same as in java.awt)
  * @return true to signal successful handling
  * @see java.awt.Event
  public boolean handleEvent (Position p, Event e) {
    if (e.id == Document.GET_UNDO) {
      e.target = this;
      return true;
      // true means do not put the message to next handlers
    } else {
      return false;
    }
  }
  */
  
}

