/**
 * Xelfi
 */

package xelfi.top.toolbar;

import java.awt.*;
import xelfi.awt.ImageButton;
import xelfi.top.MainMenu;
import xelfi.top.TopLevel;

/** ToolbarButton class is used in main toolbar of Xelfi.
 *
 * @version 	1.2 21 Jan 1997
 * @author		Petr Hamernik
 */
public class ToolbarButton extends ImageButton {
	private String hint = null;
	private static String origHint = null;
	private boolean disabled = false;
	
	/** Construct new toolbar button with image and hint.
	 * @param image Filename of icon file.
	 * @param aHint Text hint which is showed in title of
	 *			main Xelfi window.
	 */
	public ToolbarButton(Image image, String aHint) {
		super(image);
		hint = aHint;
		if (origHint == null)
			origHint = TopLevel.getMenu().getTitle();
	}
	
	public synchronized boolean mouseEnter(Event evt, int x, int y)  {
		if (hint != null)
			TopLevel.getMenu().setTitle(origHint+" ["+hint+"]");
		return super.mouseEnter(evt, x, y);
	}
	
	public synchronized boolean mouseExit(Event evt, int x, int y) {
		if ((hint != null) && (origHint != null)) {
			if (TopLevel.getMenu().getTitle().compareTo(origHint+" ["+hint+"]") == 0)
				TopLevel.getMenu().setTitle(origHint);
		}
		return super.mouseExit(evt, x, y);
	}

	public boolean mouseDown(Event event, int x, int y) {
		if (disabled)
			return true;
		else 
			return super.mouseDown(event, x, y);
	}

	public boolean mouseUp(Event event, int x, int y) {
		if (disabled)
			return true;
		else 
			return super.mouseUp(event, x, y);
	}

	public synchronized void disableButton() {
		disabled = true;
	}

	public void enableButton(boolean cond) {
		disabled = !cond;
	}

	public void enableButton() {
		disabled = false;
	}

	public boolean isEnabledButton() {
		return !disabled;
	}
}


