/** Row layout manager
* @author: Filip Dvorak
* @version: 1.0
*/

package xelfi.top.visual;

import java.awt.*;
import java.util.*;

  /** This layout layout components. Each component occupies one row. */
public class RowLayout extends LayoutBase
{
  public void layoutContainer(Container parent)
  {
    int container = parent.size().height, last = components.size() - 1, i;
    Insets conti = parent.insets();
    Dimension prefsize[] = prefSizes();
      // try ideal size
    int rows[] = preferredHeight(prefsize), top[] = new int[last + 1], bot[] = new int[last + 1];
    int dif = container - conti.top - conti.bottom - rows[last + 1] ;  // total height
    if(dif >= 0) distributeSpace(dif, rows, top, bot);  // more space than we need
    else
    {  // too little space
      Dimension minsize[] = minSizes();
      rows = minimumHeight(minsize);
      dif = container - conti.top - conti.bottom - rows[last + 1];  // total height
      if(dif > 0) distributeSpaceSpecial(dif, rows, minsize, prefsize);
      for(i = 0; i <+ last; i++) top[i] = bot[i] = 0;
    }

    int y = conti.top, x = conti.left, contwidth = parent.size().width - conti.right - conti.left, compwidth;
    Insets compi = null;
    LayoutPropertiesBase pr = null;
    for(i = 0; i <= last; i++)
    {
        // top space
      y += top[i];
        // component
      pr = (LayoutPropertiesBase)prop.elementAt(i);
      compi= pr.compInsets;
      x = conti.left + compi.left;
      compwidth = contwidth - compi.left - compi.right;
      if((dif = compwidth - prefsize[i].width) > 0)
      {                                      // distribute space horizontally
        int l = (int)(pr.leftSpaceGrowth * dif);
        x += l;
        compwidth -= l;
        compwidth -= (int)(pr.rightSpaceGrowth * dif);
      }
      ((Component)components.elementAt(i)).reshape(x, y + compi.top, compwidth, rows[i] - compi.top - compi.bottom);

        // component + bottom space
      y += rows[i] + bot[i];
    }
  }

  public Dimension minimumLayoutSize(Container  parent)
  {
    int width = 0;
    Dimension minsize[] = minSizes();
      // count max width
    int last = components.size(), i;
    Insets compi = null;
    for(i = 0; i < last; i++)
    {
      compi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      width = Math.max(minsize[i].width + compi.left + compi.right, width);
    }
      // count height
    int rows[] = minimumHeight(minsize);
    Insets ins = parent.insets();
    return new Dimension(width + ins.left + ins.right, rows[last] + ins.top + ins.bottom);
  }

  public Dimension preferredLayoutSize(Container  parent)
  {
    int width = 0;
    Dimension prefsize[] = prefSizes();
      // count max width
    int last = components.size(), i;
    Insets compi = null;
    for(i = 0; i < last; i++)
    {
      compi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      width = Math.max(prefsize[i].width + compi.left + compi.right, width);
    }
      // count height
    int rows[] = preferredHeight(prefsize);
    Insets ins = parent.insets();
    Dimension d = new Dimension(width + ins.left + ins.right, rows[last] + ins.top + ins.bottom);
    return d;

  }

    /** Builds an array of rows. Last field is total height. Array contains height of rows.
      Input parameter is an array of preferred sizes of  components.
    */
  protected int[] preferredHeight(Dimension prefsize[])
  {
    Component pom = null;
    Insets pomi = null;
    int l = components.size(), i, total = 0;
    int rows[] = new int[l + 1];
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pomi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      total += (rows[i] = prefsize[i].height + pomi.top + pomi.bottom);
    }
    rows[l] = total;
    return rows;
  }

    /** Builds an array of rows. Last field is total height. Array contains height of rows.
      Input parameter is an array of minimum sizes of  components.
    */
  protected int[] minimumHeight(Dimension minsize[])
  {
    Component pom = null;
    Insets pomi = null;
    int l = components.size(), i, total = 0;
    int rows[] = new int[l + 1];
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pomi= ((LayoutPropertiesBase)prop.elementAt(i)).compInsets;
      total += (rows[i] = minsize[i].height + pomi.top + pomi.bottom);
    }
    rows[l] = total;
    return rows;
  }

    /** Builds an array of rows. Last field is total height. Array contains height of rows. */
  protected int[] minimumHeight()
  {
    return minimumHeight(minSizes());
  }

    /** Distributes space also into top and bottom row space. */
  protected void distributeSpace(int space, int rows[], int topSpace[], int botSpace[])
  {
    Component pom = null;
    LayoutPropertiesBase pr = null;
    int l = components.size(), i;
    for(i = 0; i < l; i++)
    {
      pom = (Component)components.elementAt(i);
      pr = (LayoutPropertiesBase)prop.elementAt(i);
      rows[i] += space * pr.compGrowth;
      topSpace[i] = (int)(space * pr.topSpaceGrowth);
      botSpace[i] = (int)(space * pr.bottomSpaceGrowth);
    }
  }

    /** Distributes space of dif pixels among rows in special way. Called when preferred size is too big for a container
      and components must be resized to its minimum sizes. The extra space is distributed according to how much space is missing
      for each component to rich preferred size. */
  protected void distributeSpaceSpecial(int dif, int rows[], Dimension minsize[], Dimension prefsize[])
  {
    float cutRatio[] = new float[components.size()];
    float cutTotal = 0;
    int l = components.size(), i;
    for(i = 0; i < l; i++)
    {
      int pref = prefsize[i].height;
      int min = minsize[i].height;
      cutRatio[i] = (pref - min) / (float)pref;
      cutTotal += cutRatio[i];
    }
    for(i = 0; i < l; i++)
    {
      Component pom = (Component)components.elementAt(i);
      rows[i] += dif * (cutRatio[i] / cutTotal);
    }
  }

}

